package com.jqtx.windows.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.extra.ssh.Sftp;
import com.jqtx.infrastructure.oss.starter.config.OssUtils;
import com.jqtx.windows.common.config.SftpConfig;
import com.jqtx.windows.common.config.snow.SnowSequenceHelper;
import com.jqtx.windows.component.BackedDetailComponent;
import com.jqtx.windows.component.LoanComponent;
import com.jqtx.windows.component.LoanPlanComponent;
import com.jqtx.windows.component.LoanRepayComponent;
import com.jqtx.windows.component.enums.LoanStatusEnum;
import com.jqtx.windows.component.impl.BackedBurdenComponentImpl;
import com.jqtx.windows.component.model.*;
import com.jqtx.windows.infrastructure.feign.enums.DebtInfoStatusEnum;
import com.jqtx.windows.infrastructure.feign.enums.DebtPlanStatusEnum;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Component
@Slf4j
public class BackedBurdenJob {
    @Autowired
    private OssUtils ossUtils;

    @Autowired
    private LoanPlanComponent loanPlanComponent;

    @Autowired
    private BackedBurdenComponentImpl backedBurdenComponent;

    @Autowired
    private LoanComponent loanComponent;

    @Autowired
    private LoanRepayComponent loanRepayComponent;

    @Autowired
    private BackedDetailComponent backedDetailComponent;

    @Autowired
    private SftpConfig sftpConfig;


    @XxlJob("backedBurdenJob")
    public void run() {
        String param = XxlJobHelper.getJobParam();
        Sftp sftp = new Sftp(sftpConfig.getSshHost(), sftpConfig.getSshPort(), sftpConfig.getSshUser(), sftpConfig.getSshPass(), CharsetUtil.CHARSET_UTF_8);
        try {
            String date = "";
            if (StringUtils.isNotEmpty(param)) {
                date = param;
            } else {
                Calendar cal = Calendar.getInstance();
                //获取前面的时间用-负号
                cal.setTime(new Date());
                cal.add(Calendar.DAY_OF_MONTH, -1);
                Date time = cal.getTime();
                date = DateUtil.format(time, "yyyyMMdd");
            }
            if (sftp.isDir("/download/assurance/" + date)) {
                sftp.cd("/download/assurance/" + date);
                sftp.get("assurance_" + date + ".txt", "/assurance_sftp.txt");
                ossUtils.ossUpload("ZN/assurance/" + "assurance_" + date + ".txt", FileUtil.file("/assurance_sftp.txt"));
                String stringObject = getStringObject("ZN/assurance/" + "assurance_" + date + ".txt");
                log.info("oss融担数据:{}", stringObject);
                //存储并去平账
                List<BurdenModel> getlist = getlist(stringObject);
                for (BurdenModel model : getlist) {
                    if (new BigDecimal(model.getPepayBurden()).compareTo(BigDecimal.ZERO) > 0) {
                        backedBurdenComponent.insert(model);
                        backedBurden(model);
                    }
                }
                FileUtil.del("/assurance_sftp.txt");
            } else {
                log.info("浙农融担文件不存在:{}", date);
            }
        } catch (Exception e) {
            log.info("浙农代偿错误:{}", e.getMessage());
        } finally {
            //关闭sftp连接
            sftp.close();
        }
    }

    public String getStringObject(String uri) {
        byte[] bytes = new byte[0];
        try {
            bytes = ossUtils.ossDownLoad(uri);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return new String(bytes);
    }


    /**
     * 组装还融担的全部model
     *
     * @param jsonResult
     * @return
     */
    private List<BurdenModel> getlist(String jsonResult) {
        ArrayList<BurdenModel> list = new ArrayList<>();
        String[] split = jsonResult.split("\n");
        if (split == null) {
            return list;
        }
        for (String modelstr : split) {
            String[] split1 = modelstr.split("\\|");
            BurdenModel model = new BurdenModel();
            model.setLoanNo(split1[0]);
            model.setRepayType(split1[1]);
            model.setRepayDate(split1[2]);
            model.setPlanIndex(split1[3]);
            model.setPepayBurden(split1[4]);
            model.setPlanNo(getPlanNo(split1[0], split1[3]));
            //model.setMemo(split1[0]);
            list.add(model);
        }
        return list;
    }

    /**
     * 获取planNo
     *
     * @param loanNo
     * @param planIndex
     * @return
     */
    private String getPlanNo(String loanNo, String planIndex) {
        LoanPlanModel planModel = loanPlanComponent.getPlanNoByLoan(loanNo, planIndex);
        return planModel.getPlanNo();
    }

    /**
     * 还融担
     *
     * @param model
     */
    private void backedBurden(BurdenModel model) {
        //生成融担还款编号
        String refundNo = SnowSequenceHelper.nextSequence(SnowSequenceHelper.REFUND_NO);
        //生成 windows_repay 信息
        RepayModel repayModel = creditRepay(model, refundNo);
        loanRepayComponent.insert(repayModel);
        //生成 windows_backed_detail 信息
        BackedDetailModel backedDetailModel = creditBackedDetailModel(model, refundNo);
        backedDetailComponent.insert(backedDetailModel);
        //修改 windows_plan backAmt 信息 还款状态
        upWindowsPlan(model);
        //修改 windows_loan 还款状态
        LoanModel loanModel = loanComponent.getByLoanNo(model.getLoanNo());
        upWindowsLoan(loanModel);
    }

    /**
     * 生成 windows_repay
     *
     * @param model
     * @return
     */
    public RepayModel creditRepay(BurdenModel model, String refundNo) {
        RepayModel windowsRepay = new RepayModel();
        windowsRepay.setLoanNo(model.getLoanNo());
        windowsRepay.setBackedNo(refundNo);
        windowsRepay.setBackedAmt(new BigDecimal(model.getPepayBurden()));
        windowsRepay.setIsSettle(0);
        windowsRepay.setBackedType("BURDEN");
        windowsRepay.setRepayResult("SUCCESS");
        windowsRepay.setRepayStatus("FINISHED");
        windowsRepay.setCreator("浙农");
        windowsRepay.setGmtCreated(LocalDateTime.now());
        windowsRepay.setGmtModified(LocalDateTime.now());
        return windowsRepay;
    }

    /**
     * 生成 windows_backed_detail
     *
     * @param model
     * @return
     */
    public BackedDetailModel creditBackedDetailModel(BurdenModel model, String refundNo) {
        BackedDetailModel debtBackedModel = new BackedDetailModel();
        debtBackedModel.setBackedNo(refundNo);
        debtBackedModel.setPlanNo(model.getPlanNo());
        debtBackedModel.setBackedDate(LocalDate.now());
        debtBackedModel.setBackedAmt(new BigDecimal(model.getPepayBurden()));
        debtBackedModel.setBackedBen(BigDecimal.ZERO);
        debtBackedModel.setBackedXi(BigDecimal.ZERO);
        debtBackedModel.setBackedFu(BigDecimal.ZERO);
        debtBackedModel.setBackedFa(BigDecimal.ZERO);
        debtBackedModel.setBackedBurden(new BigDecimal(model.getPepayBurden()));
        debtBackedModel.setBackedSpeaccount(BigDecimal.ZERO);
        debtBackedModel.setBackedBreak(BigDecimal.ZERO);
        debtBackedModel.setBackedType("BURDEN");
        debtBackedModel.setGmtCreated(LocalDateTime.now());
        debtBackedModel.setGmtModified(LocalDateTime.now());
        return debtBackedModel;
    }


    /**
     * 生成 windows_backed_detail
     *
     * @param model
     * @return
     */
    public void upWindowsPlan(BurdenModel model) {
        LoanPlanModel planModel = loanPlanComponent.getPlanNoByLoan(model.getLoanNo(), model.getPlanIndex());
        planModel.setBackedBurden(new BigDecimal(model.getPepayBurden()));
        BigDecimal planAmt = planModel.getPlanBen().add(planModel.getPlanXi()).add(planModel.getPlanFu()).add(planModel.getPlanFa()).add(planModel.getPlanBurden());
        BigDecimal backedAmt = planModel.getBackedBen().add(planModel.getBackedXi()).add(planModel.getBackedFu()).add(planModel.getBackedFa()).add(planModel.getBackedBurden());
        if (!(planAmt.compareTo(backedAmt) > 0)) {
            planModel.setStatus(LoanStatusEnum.SETTLED.getCode());
        }
        loanPlanComponent.updateByLoanPlanModel(planModel);
    }

    private void upWindowsLoan(LoanModel loanModel) {
        List<LoanPlanModel> planList = loanPlanComponent.getByLoan(loanModel.getLoanNo());
        //存在还款中计划改为还款中
        List<LoanPlanModel> backingList = planList.stream().filter(i -> i.getStatus() == DebtPlanStatusEnum.BACKING.getCode()).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(backingList)) {
            loanModel.setLoanStatus(DebtInfoStatusEnum.SETTLED.getCode());
        }
        //修改 windows_loan 还款状态
        loanComponent.updateByLoanNo(loanModel);
    }

}
